@@ -40,6 +40,7 @@
 #include "options/m_config.h"
 #include "common/msg.h"
 #include "common/global.h"
+#include "common/stats.h"
 #include "video/hwdec.h"
 #include "video/mp_image.h"
 #include "sub/osd.h"
@@ -51,6 +52,7 @@ extern const struct vo_driver video_out_x11;
 extern const struct vo_driver video_out_vdpau;
 extern const struct vo_driver video_out_xv;
 extern const struct vo_driver video_out_gpu;
+extern const struct vo_driver video_out_gpu_next;
 extern const struct vo_driver video_out_libmpv;
 extern const struct vo_driver video_out_null;
 extern const struct vo_driver video_out_image;
@@ -60,9 +62,11 @@ extern const struct vo_driver video_out_drm;
 extern const struct vo_driver video_out_direct3d;
 extern const struct vo_driver video_out_sdl;
 extern const struct vo_driver video_out_vaapi;
+extern const struct vo_driver video_out_dmabuf_wayland;
 extern const struct vo_driver video_out_wlshm;
 extern const struct vo_driver video_out_rpi;
 extern const struct vo_driver video_out_tct;
+extern const struct vo_driver video_out_sixel;
 
 const struct vo_driver *const video_out_drivers[] =
 {
@@ -71,6 +75,9 @@ const struct vo_driver *const video_out_drivers[] =
     &video_out_mediacodec_embed,
 #endif
     &video_out_gpu,
+#if HAVE_LIBPLACEBO_NEXT
+    &video_out_gpu_next,
+#endif
 #if HAVE_VDPAU
     &video_out_vdpau,
 #endif
@@ -86,6 +93,9 @@ const struct vo_driver *const video_out_drivers[] =
 #if HAVE_SDL2_VIDEO
     &video_out_sdl,
 #endif
+#if HAVE_DMABUF_WAYLAND
+    &video_out_dmabuf_wayland,
+#endif
 #if HAVE_VAAPI_X11 && HAVE_GPL
     &video_out_vaapi,
 #endif
@@ -104,6 +114,9 @@ const struct vo_driver *const video_out_drivers[] =
 #endif
 #if HAVE_RPI_MMAL
     &video_out_rpi,
+#endif
+#if HAVE_SIXEL
+    &video_out_sixel,
 #endif
     &video_out_lavc,
     NULL
@@ -127,6 +140,7 @@ struct vo_internal {
     bool want_redraw;               // redraw request from VO to player
     bool send_reset;                // send VOCTRL_RESET
     bool paused;
+    bool wakeup_on_done;
     int queued_events;              // event mask for the user
     int internal_events;            // event mask for us
 
@@ -162,6 +176,8 @@ struct vo_internal {
 
     double display_fps;
     double reported_display_fps;
+
+    struct stats_ctx *stats;
 };
 
 extern const struct m_sub_options gl_video_conf;
@@ -199,7 +215,6 @@ const struct m_obj_list vo_obj_list = {
         {"opengl-cb", "libmpv"},
         {0}
     },
-    .allow_unknown_entries = true,
     .allow_trailer = true,
     .disallow_positional_parameters = true,
     .use_global_options = true,
@@ -294,6 +309,7 @@ static struct vo *vo_create(bool probing, struct mpv_global *global,
         .dispatch = mp_dispatch_create(vo),
         .req_frames = 1,
         .estimated_vsync_jitter = -1,
+        .stats = stats_ctx_create(vo, global, "vo"),
     };
     mp_dispatch_set_wakeup_fn(vo->in->dispatch, dispatch_wakeup_cb, vo);
     pthread_mutex_init(&vo->in->lock, NULL);
@@ -666,6 +682,7 @@ void vo_control_async(struct vo *vo, int request, void *data)
         break;
     case VOCTRL_KILL_SCREENSAVER:
     case VOCTRL_RESTORE_SCREENSAVER:
+    case VOCTRL_OSD_CHANGED:
         break;
     default:
         abort(); // requires explicit support
@@ -743,6 +760,41 @@ void vo_wakeup(struct vo *vo)
     pthread_mutex_unlock(&in->lock);
 }
 
+static bool still_displaying(struct vo *vo)
+{
+    struct vo_internal *in = vo->in;
+    int64_t now = mp_time_us();
+    int64_t frame_end = 0;
+    if (in->current_frame) {
+        frame_end = in->current_frame->pts + MPMAX(in->current_frame->duration, 0);
+        if (in->current_frame->display_synced)
+            frame_end = in->current_frame->num_vsyncs > 0 ? INT64_MAX : 0;
+    }
+    return (now < frame_end || in->rendering || in->frame_queued) && in->hasframe;
+}
+
+// Return true if there is still a frame being displayed (or queued).
+bool vo_still_displaying(struct vo *vo)
+{
+    pthread_mutex_lock(&vo->in->lock);
+    bool res = still_displaying(vo);
+    pthread_mutex_unlock(&vo->in->lock);
+    return res;
+}
+
+// Make vo issue a wakeup once vo_still_displaying() becomes true.
+void vo_request_wakeup_on_done(struct vo *vo)
+{
+    struct vo_internal *in = vo->in;
+    pthread_mutex_lock(&vo->in->lock);
+    if (still_displaying(vo)) {
+        in->wakeup_on_done = true;
+    } else {
+        wakeup_core(vo);
+    }
+    pthread_mutex_unlock(&vo->in->lock);
+}
+
 // Whether vo_queue_frame() can be called. If the VO is not ready yet, the
 // function will return false, and the VO will call the wakeup callback once
 // it's ready.
@@ -829,7 +881,7 @@ static bool render_frame(struct vo *vo)
 {
     struct vo_internal *in = vo->in;
     struct vo_frame *frame = NULL;
-    bool got_frame = false;
+    bool more_frames = false;
 
     update_display_fps(vo);
 
@@ -893,14 +945,21 @@ static bool render_frame(struct vo *vo)
 
     if (in->dropped_frame) {
         in->drop_count += 1;
+        wakeup_core(vo);
     } else {
         in->rendering = true;
         in->hasframe_rendered = true;
         int64_t prev_drop_count = vo->in->drop_count;
+        // Can the core queue new video now? Non-display-sync uses a separate
+        // timer instead, but possibly benefits from preparing a frame early.
+        bool can_queue = !in->frame_queued &&
+            (in->current_frame->num_vsyncs < 1 || !use_vsync);
         pthread_mutex_unlock(&in->lock);
-        wakeup_core(vo); // core can queue new video now
 
-        MP_STATS(vo, "start video-draw");
+        if (can_queue)
+            wakeup_core(vo);
+
+        stats_time_start(in->stats, "video-draw");
 
         if (vo->driver->draw_frame) {
             vo->driver->draw_frame(vo, frame);
@@ -908,11 +967,11 @@ static bool render_frame(struct vo *vo)
             vo->driver->draw_image(vo, mp_image_new_ref(frame->current));
         }
 
-        MP_STATS(vo, "end video-draw");
+        stats_time_end(in->stats, "video-draw");
 
         wait_until(vo, target);
 
-        MP_STATS(vo, "start video-flip");
+        stats_time_start(in->stats, "video-flip");
 
         vo->driver->flip_page(vo);
 
@@ -927,7 +986,7 @@ static bool render_frame(struct vo *vo)
         if (vsync.last_queue_display_time < 0)
             vsync.last_queue_display_time = mp_time_us();
 
-        MP_STATS(vo, "end video-flip");
+        stats_time_end(in->stats, "video-flip");
 
         pthread_mutex_lock(&in->lock);
         in->dropped_frame = prev_drop_count < vo->in->drop_count;
@@ -947,15 +1006,24 @@ static bool render_frame(struct vo *vo)
         in->request_redraw = false;
     }
 
-    pthread_cond_broadcast(&in->wakeup); // for vo_wait_frame()
-    wakeup_core(vo);
+    if (in->current_frame && in->current_frame->num_vsyncs &&
+        in->current_frame->display_synced)
+        more_frames = true;
 
-    got_frame = true;
+    if (in->frame_queued && in->frame_queued->display_synced)
+        more_frames = true;
+
+    pthread_cond_broadcast(&in->wakeup); // for vo_wait_frame()
 
 done:
     talloc_free(frame);
+    if (in->wakeup_on_done && !still_displaying(vo)) {
+        in->wakeup_on_done = false;
+        wakeup_core(vo);
+    }
     pthread_mutex_unlock(&in->lock);
-    return got_frame || (in->frame_queued && in->frame_queued->display_synced);
+
+    return more_frames;
 }
 
 static void do_redraw(struct vo *vo)
@@ -1030,6 +1098,7 @@ static void *vo_thread(void *ptr)
         mp_dispatch_queue_process(vo->in->dispatch, 0);
         if (in->terminate)
             break;
+        stats_event(in->stats, "iterations");
         vo->driver->control(vo, VOCTRL_CHECK_EVENTS, NULL);
         bool working = render_frame(vo);
         int64_t now = mp_time_us();
@@ -1045,10 +1114,10 @@ static void *vo_thread(void *ptr)
             }
         }
         if (vo->want_redraw && !in->want_redraw) {
-            vo->want_redraw = false;
             in->want_redraw = true;
             wakeup_core(vo);
         }
+        vo->want_redraw = false;
         bool redraw = in->request_redraw;
         bool send_reset = in->send_reset;
         in->send_reset = false;
@@ -1142,24 +1211,6 @@ void vo_seek_reset(struct vo *vo)
     pthread_mutex_unlock(&in->lock);
 }
 
-// Return true if there is still a frame being displayed (or queued).
-// If this returns true, a wakeup some time in the future is guaranteed.
-bool vo_still_displaying(struct vo *vo)
-{
-    struct vo_internal *in = vo->in;
-    pthread_mutex_lock(&vo->in->lock);
-    int64_t now = mp_time_us();
-    int64_t frame_end = 0;
-    if (in->current_frame) {
-        frame_end = in->current_frame->pts + MPMAX(in->current_frame->duration, 0);
-        if (in->current_frame->display_synced)
-            frame_end = in->current_frame->num_vsyncs > 0 ? INT64_MAX : 0;
-    }
-    bool working = now < frame_end || in->rendering || in->frame_queued;
-    pthread_mutex_unlock(&vo->in->lock);
-    return working && in->hasframe;
-}
-
 // Whether at least 1 frame was queued or rendered since last seek or reconfig.
 bool vo_has_frame(struct vo *vo)
 {
